﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.Entity;
using SerrateTFC.Domain.Entities;
using SerrateTFC.Domain.ValueObjects;
using SerrateTFC.Domain.Enums;
using SerrateTFC.Domain.Data;

namespace SerrateTFC.Infrastructure.Data
{
    /// <summary>
    /// Implementació del contracte UnitOfWork
    /// Més informació: http://msdn.microsoft.com/en-us/magazine/dd882510.aspx
    /// </summary>
    public class MainUnitOfWork : DbContext, IMainUnitOfWork
    {
        public MainUnitOfWork()
            : base("SerrateTFC")
        {
            this.Configuration.LazyLoadingEnabled = false;
        }

        public IDbSet<Customer> Customers { get; set; }
        public IDbSet<User> Users { get; set; }
        public IDbSet<Project> Projects { get; set; }
        public IDbSet<Task> Tasks { get; set; }

        public void Update<T>(T entity) where T : class, IEntity
        {
            this.Entry(entity).State = System.Data.EntityState.Modified;
        }

        public void Detach<T>(T entity) where T : class, IEntity
        {
            this.Entry(entity).State = System.Data.EntityState.Detached;
        }

        protected override void OnModelCreating(DbModelBuilder modelBuilder)
        {
            ConfigureCustomer(modelBuilder);

            ConfigureUser(modelBuilder);

            ConfigureProject(modelBuilder);

        }

        private void ConfigureProject(DbModelBuilder modelBuilder)
        {
            modelBuilder.Entity<Project>()
                .HasKey(x => x.Id)
                .Property(x => x.Id)
                .HasColumnName("ProjectId");

            modelBuilder.Entity<Project>()
                .Property(x => x.ProjectManagerId)
                .HasColumnName("UserId");

            modelBuilder.Entity<Task>()
                .HasKey(x => x.Id)
                .Property(x => x.Id)
                .HasColumnName("TaskId");

            modelBuilder.Entity<Task>()
                .HasRequired(x => x.Project)
                .WithMany(x => x.Tasks)
                .HasForeignKey(x => x.ProjectId)
                .WillCascadeOnDelete();

            modelBuilder.Entity<Task>()
                .HasRequired(x => x.Assigned)
                .WithOptional();

            modelBuilder.Entity<Task>()
                .Property(x => x.PriorityValue)
                .HasColumnName("Priority");

            modelBuilder.Entity<Task>()
                .Property(x => x.AssignedId)
                .HasColumnName("UserId");

        }

        private void ConfigureUser(DbModelBuilder modelBuilder)
        {
            modelBuilder.Entity<User>()
                .HasKey(x => x.Id)
                .Property(x => x.Id)
                .HasColumnName("UserId");             

            modelBuilder.Entity<User>()
                .Map<ProjectManager>(m =>
                {
                    m.Requires("UserTypeId").HasValue((int)Roles.ProjectManager);
                    m.MapInheritedProperties();
                })
                .Map<Administrator>(m =>
                {
                    m.Requires("UserTypeId").HasValue((int)Roles.Administrator);
                    m.MapInheritedProperties();
                })
                .Map<Resource>(m =>
                {
                    m.Requires("UserTypeId").HasValue((int)Roles.Resource);
                    m.Properties(p => new { HourlyRate = p.HourlyRate });
                    m.MapInheritedProperties();
                });

            modelBuilder.ComplexType<UserAccount>()
                .Property(x => x.FirstName)
                .HasColumnName("FirstName");

            modelBuilder.ComplexType<UserAccount>()
                .Property(x => x.LastName)
                .HasColumnName("LastName");

            modelBuilder.ComplexType<UserAccount>()
                .Property(x => x.Email)
                .HasColumnName("Email");
        }

        private void ConfigureCustomer(DbModelBuilder modelBuilder)
        {
            modelBuilder.Entity<Customer>()
                .HasKey(x => x.Id)
                .Property(x => x.Id)
                .HasColumnName("CustomerId");

            modelBuilder.ComplexType<Address>()
                .Property(x => x.PublicAddress)
                .HasColumnName("Address");

            modelBuilder.ComplexType<Address>()
                .Property(x => x.City)
                .HasColumnName("City");

            modelBuilder.ComplexType<Address>()
                .Property(x => x.PostalCode)
                .HasColumnName("PostalCode");

            modelBuilder.ComplexType<Address>()
                .Property(x => x.Country)
                .HasColumnName("Country");
        }
    }
}
