﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.Entity;
using SerrateTFC.Localization.Exceptions;
using SerrateTFC.Domain.Data;

namespace SerrateTFC.Infrastructure.Data
{
    /// <summary>
    /// Repositori base
    /// </summary>
    /// <typeparam name="T">Tipus de l'entitat</typeparam>
    public abstract class Repository<T> : IRepository<T>
        where T : class, IEntity
    {
        protected readonly IDbSet<T> _dbSet;
        protected readonly MainUnitOfWork _unitOfWork;

        public Repository(IMainUnitOfWork unitOfWork)
        {
            if (unitOfWork == null)
                throw new ArgumentNullException("unitOfWork");

            this._unitOfWork = unitOfWork as MainUnitOfWork;

            this._dbSet = this._unitOfWork.Set<T>();

            if (this._dbSet == null)
                throw new InvalidOperationException(Exceptions.DbSetNotCreated);
        }

        /// <summary>
        /// Persistir entitat
        /// </summary>
        /// <param name="entity">Instància de l'entitat</param>
        public virtual void Save(T entity)
        {
            if (entity == null)
                throw new ArgumentNullException("entity");

            if (this._dbSet.Any(e => e.Id == entity.Id))
            {
                this._unitOfWork.Update(entity);
            }
            else
            {
                this._dbSet.Add(entity);
            }
        }

        /// <summary>
        /// Hack per a entitat que han canviat de tipus
        /// per exemple: de ProjectManager a Administrator
        /// </summary>
        /// <param name="entity"></param>
        public void SaveByEntityType(T entity)
        {
            var oldType = this.GetById(entity.Id);
            
            if (oldType != null && entity.GetType() != oldType.GetType())
            {
                this.Delete(oldType);
                this._dbSet.Add(entity);
            }
            else
            {
                if (oldType != null)
                    this._unitOfWork.Detach(oldType);
                this.Save(entity);
            }
        }

        /// <summary>
        /// Elimina una entitat
        /// </summary>
        /// <param name="entity">Instància de l'entitat</param>
        public virtual void Delete(T entity)
        {
            if (entity == null)
                throw new ArgumentNullException("entity");

            this._dbSet.Remove(entity);
        }

        /// <summary>
        /// Obtè una entitat pel seu identificador
        /// </summary>
        /// <param name="id">Identificador de l'entitat</param>
        /// <returns>Instància de l'entitat</returns>
        public virtual T GetById(Guid id)
        {
            return this._dbSet.Find(id);
        }

        /// <summary>
        /// Retorna totes les entitats
        /// </summary>
        /// <returns>Totes les entitats</returns>
        public virtual IEnumerable<T> GetAll()
        {
            return this._dbSet.ToList();
        }

        /// <summary>
        /// Cerca enitats mitjançant una expressió lambda
        /// </summary>
        /// <param name="where">Expressió lambda</param>
        /// <returns>Llistat de les entitats</returns>
        public IEnumerable<T> Find(Func<T, bool> where)
        {
            return this._dbSet.Where(where).ToList();
        }
    }
}
