﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.ServiceModel.Web;
using System.Text;
using SerrateTFC.RestService.Dto;
using SerrateTFC.Domain.DomainServices;
using Microsoft.Practices.ServiceLocation;
using AutoMapper;
using SerrateTFC.Domain.Entities;
using SerrateTFC.Domain.Enums;
using SerrateTFC.Domain.Data;

namespace SerrateTFC.RestService
{
    public class MainService : IMainService
    {
        private readonly IUserDomainService _userDomainService;
        private readonly ITaskDomainService _taskDomainService;
        private readonly IProjectDomainService _projectDomainService;
        private readonly IUnitOfWork _unitOfWork;

        public MainService()
        {
            this._userDomainService = ServiceLocator.Current.GetInstance<IUserDomainService>();
            this._taskDomainService = ServiceLocator.Current.GetInstance<ITaskDomainService>();
            this._projectDomainService = ServiceLocator.Current.GetInstance<IProjectDomainService>();
            this._unitOfWork = ServiceLocator.Current.GetInstance<IUnitOfWork>();

            Mapper.CreateMap<User, UserResponse>()
                .ForMember(dest => dest.ResourceType, opt =>
                    opt.ResolveUsing<ResourceTypeResolver>());

            Mapper.CreateMap<Task, TaskItem>();

            Mapper.CreateMap<Task, TaskDetails>()
                .ForMember(dest => dest.Priority, opt =>
                    opt.MapFrom(src => src.PriorityValue));

            Mapper.CreateMap<TaskDetails, Task>()
                .ForMember(dest => dest.PriorityValue, opt =>
                    opt.MapFrom(src => src.Priority));

            Mapper.CreateMap<Project, KeyValue>();
            Mapper.CreateMap<Resource, KeyValue>()
                .ForMember(dest => dest.Name, opt =>
                    opt.MapFrom(src => src.UserAccount.ToString()));
        }

        public UserResponse LogOn(UserRequest data)
        {
            if (this._userDomainService.ValidateUser(data.UserName, data.Password))
            {
                var user = this._userDomainService.GetUserByUsername(data.UserName);
                var userData = Mapper.Map<User, UserResponse>(user);

                return userData;
            }

            return null;
        }

        public List<TaskItem> GetTasks()
        {
            // deshabilitar cache
            WebOperationContext.Current.OutgoingResponse.Headers.Add("Cache-Control", "no-cache");

            var tasks = _taskDomainService.GetAll();
            var taskItems = Mapper.Map<IEnumerable<Task>, IEnumerable<TaskItem>>(tasks);

            return taskItems.ToList();
        }


        public TaskDetails GetTask(string id)
        {
            // deshabilitar cache
            WebOperationContext.Current.OutgoingResponse.Headers.Add("Cache-Control", "no-cache");

            var task = _taskDomainService.GetById(new Guid(id));

            var taskDetails = Mapper.Map<Task, TaskDetails>(task);

            return taskDetails;
        }


        public List<KeyValue> GetProjects()
        {
            var projects = _projectDomainService.GetAll();
            var projectItems = Mapper.Map<IEnumerable<Project>, IEnumerable<KeyValue>>(projects);

            return projectItems.ToList();
        }

        public List<KeyValue> GetResources()
        {
            var resources = _userDomainService.GetAll().OfType<Resource>();
            var resourceItems = Mapper.Map<IEnumerable<Resource>, IEnumerable<KeyValue>>(resources);

            return resourceItems.ToList();
        }


        public void SaveTask(TaskDetails data)
        {
            var task = Mapper.Map<TaskDetails, Task>(data);

            _taskDomainService.SaveTask(task);
            _unitOfWork.SaveChanges();
        }
    }

    public class ResourceTypeResolver : ValueResolver<User, int>
    {
        protected override int ResolveCore(User source)
        {
            if (source is Administrator)
                return (int)Roles.Administrator;
            if (source is ProjectManager)
                return (int)Roles.ProjectManager;
            if (source is Resource)
                return (int)Roles.Resource;

            return 0;
        }
    }
}
