﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using SerrateTFC.Domain.DomainServices;
using SerrateTFC.Domain.Data;
using AutoMapper;
using SerrateTFC.Domain.Entities;
using SerrateTFC.Web.Models;
using Microsoft.Practices.ServiceLocation;
using SerrateTFC.Localization.Exceptions;
using SerrateTFC.Web.Extensions;
using SerrateTFC.Domain.Enums;

namespace SerrateTFC.Web.Controllers
{
    [UserTypeAuthorize(Roles.ProjectManager, Roles.Administrator)]
    public class ProjectController : BaseController
    {
        private readonly IProjectDomainService _projectDomainService;
        private readonly IUnitOfWork _unitOfWork;

        public ProjectController(
            IUnitOfWork unitOfWork,
            IProjectDomainService projectDomainService)
        {
            this._unitOfWork = unitOfWork;
            this._projectDomainService = projectDomainService;

            Mapper.CreateMap<Project, ProjectView>()
                .ForMember(dest => dest.CustomerCompanyName, opt =>
                    opt.ResolveUsing<CustomerNameResolver>());
            Mapper.CreateMap<Project, ProjectDetails>();
            Mapper.CreateMap<ProjectDetails, Project>();
        }

        public ActionResult Index()
        {
            var list = this._projectDomainService.GetAll();

            var listView = Mapper.Map<IEnumerable<Project>, IEnumerable<ProjectView>>(list);

            return View(listView);
        }

        public ActionResult Details(Guid? id)
        {
            var details = new ProjectDetails();

            if (id.HasValue)
            {
                var project = this._projectDomainService.GetById(id.Value);
                details = Mapper.Map<Project, ProjectDetails>(project);
            }
            else
            {
                details.Id = Guid.NewGuid();
            }

            details.ProjectManagers = GetProjectManagers(details.ProjectManagerId);
            details.Customers = GetCustomers(details.CustomerId);

            return View(details);
        }

        [HttpPost]
        public ActionResult Details(ProjectDetails details)
        {
            if (!ModelState.IsValid)
            {
                details.ProjectManagers = GetProjectManagers(details.ProjectManagerId);
                details.Customers = GetCustomers(details.CustomerId);
                return View(details);
            }

            var project = Mapper.Map<ProjectDetails, Project>(details);

            this._projectDomainService.SaveProject(project);

            this._unitOfWork.SaveChanges();

            return RedirectToAction("Index");
        }

        [HttpPost]
        public ActionResult Delete(Guid id)
        {
            var project = this._projectDomainService.GetById(id);
            if (project == null)
                throw new ArgumentException(Exceptions.EntityNotFound);

            this._projectDomainService.DeleteProject(project);
            this._unitOfWork.SaveChanges();

            return RedirectToAction("Index");
        }

        private IEnumerable<SelectListItem> GetCustomers(Guid customerId)
        {
            var customerService = ServiceLocator.Current.GetInstance<ICustomerDomainService>();
            var customers = customerService.GetAll();

            foreach (var item in customers)
            {
                yield return new SelectListItem()
                {
                    Text = item.CompanyName,
                    Value = item.Id.ToString(),
                    Selected = (item.Id == customerId)
                };
            }
        }

        private IEnumerable<SelectListItem> GetProjectManagers(Guid projectManagerId)
        {
            var userService = ServiceLocator.Current.GetInstance<IUserDomainService>();
            var projectManagers = userService.GetAll().OfType<ProjectManager>();

            foreach (var item in projectManagers)
            {
                yield return new SelectListItem()
                {
                    Text = item.UserAccount.ToString(),
                    Value = item.Id.ToString(),
                    Selected = (item.Id == projectManagerId)
                };
            }
        }
    }
}
