﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using SerrateTFC.Domain.DomainServices;
using SerrateTFC.Domain.Data;
using AutoMapper;
using SerrateTFC.Domain.Entities;
using SerrateTFC.Web.Models;
using SerrateTFC.Localization.Exceptions;
using Microsoft.Practices.ServiceLocation;
using SerrateTFC.Localization.Labels;
using SerrateTFC.Domain.Enums;
using SerrateTFC.Web.Extensions;

namespace SerrateTFC.Web.Controllers
{
    public class TaskController : BaseController
    {
        private readonly ITaskDomainService _taskDomainService;
        private readonly IUnitOfWork _unitOfWork;

        public TaskController(
            IUnitOfWork unitOfWork,
            ITaskDomainService taskDomainService)
        {
            this._unitOfWork = unitOfWork;
            this._taskDomainService = taskDomainService;

            Mapper.CreateMap<Task, TaskView>()
                .ForMember(dest => dest.Priority, opt =>
                    opt.MapFrom(src => Labels.ResourceManager.GetString(src.Priority.ToString())));
            Mapper.CreateMap<Task, TaskDetails>()
                .ForMember(dest => dest.ResourceId, opt =>
                    opt.MapFrom(src => src.AssignedId));
            Mapper.CreateMap<TaskDetails, Task>()
                .ForMember(dest => dest.AssignedId, opt =>
                    opt.MapFrom(src => src.ResourceId))
                .ForMember(dest => dest.Progress, opt =>
                    opt.MapFrom(src => Math.Floor(src.Progress)));
        }

        public ActionResult Index()
        {
            var list = this._taskDomainService.GetAll();

            User currentUser = GetCurrentUser();
            // filtrem si l'usuari és desenvolupador
            if (currentUser is Resource)
            {
                list = list.Where(x => x.AssignedId == currentUser.Id).ToList();
            }

            var listView = Mapper.Map<IEnumerable<Task>, IEnumerable<TaskView>>(list);

            return View(listView);
        }

        public ActionResult Details(Guid? id)
        {
            var details = new TaskDetails();

            if (id.HasValue)
            {
                var task = this._taskDomainService.GetById(id.Value);
                details = Mapper.Map<Task, TaskDetails>(task);
            }
            else
            {
                details.Id = Guid.NewGuid();
            }

            details.Projects = GetProjects(details.ProjectId);
            details.Resources = GetResources(details.ResourceId);
            details.Priorities = GetPriorities(details.Priority);

            return View(details);
        }

        [HttpPost]
        public ActionResult Details(TaskDetails details)
        {
            if (!ModelState.IsValid)
            {
                details.Projects = GetProjects(details.ProjectId);
                details.Resources = GetResources(details.ResourceId);
                details.Priorities = GetPriorities(details.Priority);
                return View(details);
            }

            var task = Mapper.Map<TaskDetails, Task>(details);

            this._taskDomainService.SaveTask(task);

            this._unitOfWork.SaveChanges();

            return RedirectToAction("Index");
        }

        [UserTypeAuthorize(Roles.ProjectManager, Roles.Administrator)]
        [HttpPost]
        public ActionResult Delete(Guid id)
        {
            var task = this._taskDomainService.GetById(id);
            if (task == null)
                throw new ArgumentException(Exceptions.EntityNotFound);

            this._taskDomainService.DeleteTask(task);
            this._unitOfWork.SaveChanges();

            return RedirectToAction("Index");
        }

        private IEnumerable<SelectListItem> GetResources(Guid resourceId)
        {
            var userService = ServiceLocator.Current.GetInstance<IUserDomainService>();
            var resources = userService.GetAll().OfType<Resource>();

            foreach (var item in resources)
            {
                yield return new SelectListItem()
                {
                    Text = item.UserAccount.ToString(),
                    Value = item.Id.ToString(),
                    Selected = (item.Id == resourceId)
                };
            }
        }

        private IEnumerable<SelectListItem> GetProjects(Guid projectId)
        {
            var projectService = ServiceLocator.Current.GetInstance<IProjectDomainService>();
            var projects = projectService.GetAll();

            foreach (var item in projects)
            {
                yield return new SelectListItem()
                {
                    Text = item.Name.ToString(),
                    Value = item.Id.ToString(),
                    Selected = (item.Id == projectId)                    
                };
            }
        }

        private IEnumerable<SelectListItem> GetPriorities(int priority)
        {
            foreach (Priority item in Enum.GetValues(typeof(Priority)))
            {
                yield return new SelectListItem()
                {
                    Text = Labels.ResourceManager.GetString(item.ToString()),
                    Value = Convert.ToString((int)item),
                    Selected = (int)item == priority
                };
            }
        }
    }
}
