﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using SerrateTFC.Domain.Data;
using SerrateTFC.Domain.DomainServices;
using SerrateTFC.Domain;
using AutoMapper;
using SerrateTFC.Web.Models;
using SerrateTFC.Domain.Entities;
using SerrateTFC.Localization.Exceptions;
using SerrateTFC.Domain.Enums;
using SerrateTFC.Localization.Labels;
using SerrateTFC.Domain.ValueObjects;
using SerrateTFC.Web.Extensions;

namespace SerrateTFC.Web.Controllers
{
    [UserTypeAuthorize(Roles.Administrator)]
    public class UserController : BaseController
    {
        private readonly IUnitOfWork _unitOfWork;
        private readonly IUserDomainService _userDomainService;

        public UserController(
            IUnitOfWork unitOfWork,
            IUserDomainService userDomainService)
        {
            this._unitOfWork = unitOfWork;
            this._userDomainService = userDomainService;

            Mapper.CreateMap<User, UserView>()
                .ForMember(src => src.ResourceType, opt =>
                    opt.ResolveUsing<ResourceNameResolver>());
            Mapper.CreateMap<User, UserDetails>()
                .ForMember(src => src.HourlyRate, opt =>
                    opt.ResolveUsing<ResourceHourlyRateResolver>())
                .ForMember(src => src.ResourceType, opt =>
                    opt.ResolveUsing<ResourceTypeResolver>())
                .ForMember(src => src.RepeatPassword, opt =>
                    opt.MapFrom(src => src.Password));
            Mapper.CreateMap<UserDetails, User>()
                .ConvertUsing<UserTypeConverter>();
        }

        public ActionResult Index()
        {
            var list = this._userDomainService.GetAll();

            var listView = Mapper.Map<IEnumerable<User>, IEnumerable<UserView>>(list);

            return View(listView);
        }

        public ActionResult Details(Guid? id)
        {
            var details = new UserDetails();            

            if (id.HasValue)
            {
                var user = this._userDomainService.GetById(id.Value);
                details = Mapper.Map<User, UserDetails>(user);
            }
            else
            {
                details.Id = Guid.NewGuid();
            }

            details.ResourceTypes = GetUserTypes(details.ResourceType);

            return View(details);
        }

        [HttpPost]
        public ActionResult Details(UserDetails details)
        {
            if (!ModelState.IsValid)
            {
                details.ResourceTypes = GetUserTypes(details.ResourceType);
                return View(details);
            }

            var user = Mapper.Map<UserDetails, User>(details);

            try
            {
                this._userDomainService.SaveUser(user);

                this._unitOfWork.SaveChanges();
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("", ex.Message);
                details.ResourceTypes = GetUserTypes(details.ResourceType);
                return View(details);
            }

            return RedirectToAction("Index");
        }

        [HttpPost]
        public ActionResult Delete(Guid id)
        {
            var user = this._userDomainService.GetById(id);
            if (user == null)
                throw new ArgumentException(Exceptions.EntityNotFound);

            this._userDomainService.DeleteUser(user);
            this._unitOfWork.SaveChanges();

            return RedirectToAction("Index");
        }

        private IEnumerable<SelectListItem> GetUserTypes(int resourceType)
        {
            yield return new SelectListItem()
            {
                Value = Convert.ToString((int)Roles.Administrator),
                Text = Labels.Administrator,
                Selected = (resourceType == (int)Roles.Administrator)
            };

            yield return new SelectListItem()
            {
                Value = Convert.ToString((int)Roles.ProjectManager),
                Text = Labels.ProjectManager,
                Selected = (resourceType == (int)Roles.Administrator)
            };

            yield return new SelectListItem()
            {
                Value = Convert.ToString((int)Roles.Resource),
                Text = Labels.Resource,
                Selected = (resourceType == (int)Roles.Administrator)
            };
        }
    }
}
