﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using AutoMapper;
using SerrateTFC.Web.Models;
using SerrateTFC.Domain.Entities;
using SerrateTFC.Domain.Enums;
using SerrateTFC.Domain.ValueObjects;
using SerrateTFC.Localization.Labels;
using Microsoft.Practices.ServiceLocation;
using SerrateTFC.Domain.DomainServices;

namespace SerrateTFC.Web.Extensions
{
    /// <summary>
    /// Converteix el model de vista a la entitat de domini corresponent
    /// </summary>
    public class UserTypeConverter : ITypeConverter<UserDetails, User>
    {
        public User Convert(ResolutionContext context)
        {
            var source = (UserDetails)context.SourceValue;
            User destination = null;
            if (source.ResourceType == (int)Roles.Administrator)
            {
                destination = Mapper.DynamicMap<UserDetails, Administrator>(source);
            }
            if (source.ResourceType == (int)Roles.ProjectManager)
            {
                destination = Mapper.DynamicMap<UserDetails, ProjectManager>(source);
            }
            if (source.ResourceType == (int)Roles.Resource)
            {
                destination = Mapper.DynamicMap<UserDetails, Resource>(source);
            }

            destination.UserAccount = new UserAccount(source.UserAccountFirstName, source.UserAccountLastName, source.UserAccountEmail);

            return destination;
        }
    }

    public class ResourceNameResolver : ValueResolver<User, string>
    {
        protected override string ResolveCore(User source)
        {
            if (source is Administrator)
                return Labels.Administrator;
            if (source is ProjectManager)
                return Labels.ProjectManager;
            if (source is Resource)
                return Labels.Resource;

            return null;
        }
    }

    public class ResourceHourlyRateResolver : ValueResolver<User, decimal>
    {
        protected override decimal ResolveCore(User source)
        {
            if (source is Resource)
                return (source as Resource).HourlyRate;

            return 0;
        }
    }

    public class ResourceTypeResolver : ValueResolver<User, int>
    {
        protected override int ResolveCore(User source)
        {
            if (source is Administrator)
                return (int)Roles.Administrator;
            if (source is ProjectManager)
                return (int)Roles.ProjectManager;
            if (source is Resource)
                return (int)Roles.Resource;

            return 0;
        }
    }

    public class CustomerResolver : ValueResolver<ProjectDetails, Customer>
    {
        protected override Customer ResolveCore(ProjectDetails source)
        {
            var customerDomainService = ServiceLocator.Current.GetInstance<ICustomerDomainService>();

            return customerDomainService.GetById(source.CustomerId);
        }
    }

    public class ProjectManagerResolver : ValueResolver<ProjectDetails, ProjectManager>
    {
        protected override ProjectManager ResolveCore(ProjectDetails source)
        {
            var userDomainService = ServiceLocator.Current.GetInstance<IUserDomainService>();

            return userDomainService.GetById(source.ProjectManagerId) as ProjectManager;
        }
    }

    public class CustomerNameResolver : ValueResolver<Project, string>
    {
        protected override string ResolveCore(Project source)
        {
            var customerDomainService = ServiceLocator.Current.GetInstance<ICustomerDomainService>();

            var customer = customerDomainService.GetById(source.CustomerId);

            return customer.CompanyName;
        }
    }
}