﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using Microsoft.Practices.Unity;
using System.Reflection;
using SerrateTFC.Domain.DomainServices;
using SerrateTFC.Domain.Repositories;
using SerrateTFC.Infrastructure.Data.Repositories;
using SerrateTFC.Infrastructure.Data;
using SerrateTFC.Domain.Data;
using Microsoft.Practices.ServiceLocation;

namespace SerrateTFC.Web.Unity
{
    /// <summary>
    /// Classe que implementa IDependencyResolver per simplificar el patró Inversion of Control
    /// </summary>
    public class UnityDependencyResolver : IDependencyResolver
    {
        private UnityContainer _container;

        public UnityDependencyResolver()
        {
            this._container = new UnityContainer();

            this.RegisterApplicationTypes();

            this.RegisterControllers();

            UnityServiceLocator locator = new UnityServiceLocator(_container);
            ServiceLocator.SetLocatorProvider(() => locator);
        }

        /// <summary>
        /// Registrem els serveis de domini i els repositoris
        /// </summary>
        private void RegisterApplicationTypes()
        {
            this._container.RegisterType<MainUnitOfWork>(new HttpContextLifetimeManager<MainUnitOfWork>());
            this._container.RegisterType<IMainUnitOfWork, MainUnitOfWork>();
            this._container.RegisterType<IUnitOfWork, MainUnitOfWork>();


            this._container.RegisterType<ICustomerDomainService, CustomerDomainService>();
            this._container.RegisterType<ICustomerRepository, CustomerRepository>();
            this._container.RegisterType<IUserDomainService, UserDomainService>();
            this._container.RegisterType<IUserRepository, UserRepository>();
            this._container.RegisterType<IProjectRepository, ProjectRepository>();
            this._container.RegisterType<IProjectDomainService, ProjectDomainService>();
            this._container.RegisterType<ITaskRepository, TaskRepository>();
            this._container.RegisterType<ITaskDomainService, TaskDomainService>();
        }

        /// <summary>
        /// Registrem per reflection tots els Controllers de l'aplicació
        /// </summary>
        private void RegisterControllers()
        {
            Assembly assembly = Assembly.GetExecutingAssembly();
            IEnumerable<Type> controllers = assembly.GetExportedTypes()
                .Where(x => typeof(IController).IsAssignableFrom(x) && !x.IsAbstract);

            foreach (Type controllerType in controllers)
            {
                this._container.RegisterType(controllerType);
            }
        }

        public object GetService(Type serviceType)
        {
            try
            {
                return this._container.Resolve(serviceType);
            }
            catch
            {
                return null;
            }
        }

        public IEnumerable<object> GetServices(Type serviceType)
        {
            try
            {
                return this._container.ResolveAll(serviceType);
            }
            catch
            {
                return null;
            }
        }
    }
}